{ Jednoduche sekvencni cteni a zapis Dat z/do souboru po blocich pres
  vyrovnavaci pamet (VP). Jednotka zajistuje praci pouze s jednim vstupnim
  (read-mod) a jednim vystupnim (write-mode) souborem.

  POZOR na spolecne pouzivani VP a primeho pristupu k souboru (napr. Seek)
}
unit VMFiles;

interface

type tData = byte;

const
   { konstanty pro zpusob prace s VP a souborem  }
   Reading = 0;     { pro cteni }
   Writing = 1;     { pro zapis }

var InF, OutF: file; { globalni vstupni a vystupni soubor pro umozneni
                       vyjmecneho zkombinovani primeho pristupu s pristupem
                       pres VP - POZOR }

procedure VFInit;
procedure VFClose(FType: byte);
function VFSet(FName: string; FType: byte): boolean;
function VEof: boolean;
procedure VRead(var D: TData);
procedure VWrite(var D: TData);
procedure VInBufClear;

implementation

const BUFSIZE = 8192;

type tFBuf = record Buf: array [1..BUFSIZE] of tData;
                Size: integer;
                Act: word;  { prvni volna pozice }
                EoInF: boolean;
                end;

var InFinUse, OutFinUse: boolean;
    InB, OutB: tFBuf;  { I/O vyrovnavaci pameti }


function VEof: boolean;
{ Vraci TRUE pokud byl jiz precten (VRead) cely vstupni soubor. Nahrazuje tak
  standartni Eof (pro cteni). }
begin
   VEof:= InB.EoInF;
end;


procedure VRead(var D: TData);
{ Cte data z disku pres VP. Nehlida EoInF (konec souboru, resp. VP). Pokud je
  volana pri EoInF ("cteni za koncem" souboru) vraci nedefinovane (resp.
  stare) hodnoty z VP. Pro spravne cteni je potreba predem zkontolovat konec
  souboru (EoInF) pomoci funkce IsEoInF }
begin
   with InB do begin
      D:= Buf[Act];
      if Act < BUFSIZE then Act:= Act + 1
                       else begin BlockRead(InF, Buf, BUFSIZE, Size);
                                  Act:= 1;
                                  end;
       EoInF:= Act > Size;

      end;
end;


procedure VWrite(var D: TData);
{ Zapisuje data do souboru pres VP. Nekontoluje spravnost zapisu z VP na disk
  (predpoklada ze se blok podari vzdy ulozit). Na disk ukalda pouze cely blok
  VP. Nekompletni blok VP se uklada automaticky az pred uzavrenim vystupniho
  souboru (VCloseF) }
begin
   with OutB do begin
      Buf[Act]:= D;
      if Act < BUFSIZE then Act:= Act + 1
                       else begin BlockWrite(OutF, Buf, BUFSIZE, Size);
                                  Act:= 1;
                                  end;
      end;
end;


procedure VInBufClear;
{ Vyprazdneni vstupni VP. Pro kombinaci s primym pristupem (Seek a Read) }
begin
   with InB do begin
      BlockRead(InF, Buf, BUFSIZE, Size);  { hned nacte prvni blok }
      Act:= 1;
      EoInF:= Size = 0;
      end;
end;


procedure VFInit;
{ Inicializace (obou) souboru. Nutne pouzit jako prvni (ale jen jednou). }
begin
   InFinUse:= FALSE;
   OutFinUse:= FALSE;
end;


procedure VFClose(FType: byte);
{ Uzavreni vstupniho nebo vystupniho souboru. }
begin
   case FType of
      Reading : if InFinUse then begin
                   close(InF);
                   InFinUse:= FALSE;
                   end;
      Writing : if OutFinUse then with OutB do begin
                   if Act > 1 then  { vyprazdni (ulozi) VP pred uzavrenim }
                      BlockWrite(OutF, Buf, Act - 1, Size);
                   close(OutF);
                   OutFinUse:= FALSE;
                   end;
   end;
end;


function VFSet(FName: string; FType: byte): boolean;
{ Otevreni souboru. FName je nazev souboru, ktery bude pouzit pro cteni nebo
  zapis v zavislosti na FType (Reading, Writing). Pokud je jiz pouzivan jiny
  soubor bude uzavren a pouzije se novy. Funkce vraci FALSE pokud se soubor
  nepodari otevrit nebo vytvorit. }
begin
   VFClose(FType);   { nejprve uzavre stary soubor (pokud je otevren) }
   VFSet:= FALSE;    { pri neznamem FType funkce vrati chybu }
   case FType of
      Reading : begin assign(InF, FName);
                      FileMode:= 0;
                      {$I-} reset(InF, SizeOf(tData)); {$I+}
                      if IOResult = 0 then with InB do begin
                         InFinUse:= TRUE;
                         VFSet:= TRUE;
                         VInBufClear;
                         end;
                      end;
      Writing : begin assign(OutF, FName);
                      {$I-} rewrite(OutF, SizeOf(tData)); {$I+}
                      if IOResult = 0 then begin
                         OutFinUse:= TRUE;
                         VFSet:= TRUE;
                         OutB.Act:= 1;  { Pripravi VP pro prvni zapis }
                         end;
                      end;
      end;
end;


end.