{ Jednoduchy textovy editor map pro hru Boulder Man

  Umoznuje vytvaret a upravovat korektni mapy pro hru BM (mapy, ktere maji na
  okrajich "Stabilni zed" a obsahuji prave jednoho "Hrace" a jeden "Unikovy
  vychod". Vystupni soubor (vytvorena mapa) obsahuje v prvnich dvou bytech
  rozmer ulozene mapy a pak prislosnou cast matice bytu
}

uses crt, LipCrt,
     VMFiles;  { nacitani bytu po blocich pres vyrovnavaci pamet (VP) }

{$I BMCOMMIF.INC}
{$I BMCOMMIM.INC}

type TEditChar = record c : char;
                        col: byte;
                        txt: string[14];
                        end;
     TEditMap = record M: TMap;         { mapa }
                       X, Y: shortint;    { souradnice akt pozice }
                       PosX, PosY: byte;  { pozice mapy na obrazovce }
                       IsPlayer,
                       IsExit: boolean;
                       end;
const
   TITLE = 'Editor map pro hru Boulder Man';
   EArr: array [1..LastObj] of TEditChar = (
      (c : #32;  col: 7;  txt: 'Mezera'),
      (c : #177; col: 10; txt: 'Trava'),
      (c : #219; col: 6;  txt: 'Zed stabilni'),
      (c : #178; col: 6;  txt: 'Zed nestabilni'),
      (c : #69;  col: 15; txt: 'Unikovy vychod'),
      (c : #4;   col: 12; txt: 'Kamen'),
      (c : #4;   col: 11; txt: 'Diamant'),
      (c : #254; col: 14; txt: 'Trhavina'),
      (c : #1;   col: 15; txt: 'Hrac'),
      (c : #2;   col: 9;  txt: 'Prisera')
      );  { poradi odpovida konstantam objektu }


procedure InitMap(var EM: TEditMap; SizeX, SizeY: byte);
{ Vytvari prazdnou mapu, podle zadanych rozmeru SizeX, SizeY (pri prekroceni
  povolenych hodnot pozije maximalni resp. minimalni mozne. }
var i, j: byte;
begin
   with EM do begin
      if SizeX > MAXMX then M.SizeX:= MAXMX
                       else if SizeX < MINMX then M.SizeX:= MINMX;
      if SizeY > MAXMY then M.SizeY:= MAXMY
                       else if SizeY < MINMY then M.SizeY:= MINMY;
      for i:= 1 to M.SizeX do M.M[i, 1]:= ZEDS;
      for j:= 2 to M.SizeY - 1 do begin
         M.M[1, j]:= ZEDS;
         for i:= 2 to M.SizeX - 1 do M.M[i, j]:= TRAVA;
         M.M[M.SizeX, j]:= ZEDS;
         end;
      for i:= 1 to M.SizeX do M.M[i, M.SizeY]:= ZEDS;
      IsExit:= FALSE;
      IsPlayer:= FALSE;
      end;
end;


procedure MovePos(var EM: TEditMap; X, Y: ShortInt);
{ Osetruje prekroceni okraju mapy a presouva kurzor v mape relativne o X, Y
  (resp. o povolene hodnoty) }
begin
   if X > 0 then
      if EM.X + X < EM.M.SizeX then EM.X:= EM.X + X
                               else EM.X:= EM.M.SizeX - 1
   else if X < 0 then
      if EM.X + X > 1 then EM.X:= EM.X + X
                      else EM.X:= 2;
   if Y > 0 then
      if EM.Y + Y < EM.M.SizeY then EM.Y:= EM.Y + Y
                               else EM.Y:= EM.M.SizeY - 1
   else if Y < 0 then
      if EM.Y + Y > 1 then EM.Y:= EM.Y + Y
                      else EM.Y:= 2;
   gotoxy(EM.PosX + EM.X, EM.PosY + EM.Y);
end;


procedure PutItem(var EM: TEditMap; Obj: byte);
{ Zapisuje polozky do mapy ale HRACe a VYCHOD jen jednou }
begin
   with EM do
      if not((IsExit and (Obj = VYCHOD)) or
             (IsPlayer and (Obj = HRAC))) then begin
         textattr:= EArr[Obj].col;
         write(EArr[Obj].c);
         MovePos(EM, 0, 0);
         if IsExit then IsExit:= M.M[EM.X, EM.Y] <> VYCHOD;
         if IsPlayer then IsPlayer:= M.M[EM.X, EM.Y] <> HRAC;
         M.M[EM.X, EM.Y]:= Obj;
         IsExit:= IsExit or (Obj = VYCHOD);
         IsPlayer:= IsPlayer or (Obj = HRAC);
         end;
end;


procedure PrintMap(X, Y: byte; var EM: TEditMap);
{ Tiskne celou mapu a nastavuje kurzor v mape do leveho horniho rohu }
var i, j: byte;
begin
  EM.PosX:= X;  EM.PosY:= Y;
  with EM do
     for j:= 1 to MAXMY do begin
        gotoxy(PosX + 1, PosY + j);
        for i:= 1 to MAXMX do
           if (i > M.SizeX) or (j > M.SizeY)
              then begin textattr:= 8;  write(#250); end
              else begin textattr:= EArr[M.M[i, j]].col;
                         write(EArr[M.M[i, j]].c);
                         end
        end;
   EM.X:= 2;  EM.Y := 2;
end;


procedure PrintTable(X, Y, Active: byte);
{ Tiskne tabulku objektu, Active urcuje zvyraznenou polozku }
var i: byte;
begin
   for i:= 1 to LastObj do begin
      if Active = i then textattr:= 15 else textattr:= 7;
      gotoxy(X + 1, Y + i );   write(EArr[i].txt:15, ' ');
      textattr:= EArr[i].col;  write(EArr[i].c);
      end;
end;

procedure PrintInfo(X, Y: byte);
{ Tiskne popis ovladani }
begin
   textattr:= 7;
   gotoxy(X + 1, Y + 1);  write('<Sipky>         - Pohyb v mape');
   gotoxy(X + 1, Y + 2);  write('<Tab>, <.>, <,> - Zmena polozky');
   gotoxy(X + 1, Y + 3);  write('<Space>         - Vlozeni polozky');
   gotoxy(X + 1, Y + 4);  write('<Enter>         - Ulozit mapu');
   gotoxy(X + 1, Y + 5);  write('<Back Space>    - Obnovit mapu');
   gotoxy(X + 1, Y + 6);  write('<Escape>        - Konec');
   gotoxy(X + 1, Y + 8);
   write('Poznamka: Mapa musi obsahovat prave 1 Hrace a 1 Unikovy vychod');
end;


function SaveMap(M: TMap; FName: string): boolean;
{ Ulozi mapu do souboru. Nejprve rozmer mapy (2 Byty) a pak matici M.M (jen
  jeji vyuzivanou cast) zleva doprava, zdola nahoru).
  Protoze se uklada pres VP je pred prvnim volanim nutne pouzit VFInit. }
var f: file of byte;
    i, j : byte;
begin
   if VFSet(FName, Writing) then begin
      SaveMap:= TRUE;
      VWrite(M.SizeX);
      VWrite(M.SizeY);
      for j:= 1 to M.SizeY do
         for i:= 1 to M.SizeX do VWrite(M.M[i,j]);
      VFClose(Writing);
      end else SaveMap:= FALSE;
end;


procedure Msg(col: byte; txt: string; Wait: boolean);
{ Tiskne hlaseni. Pokud je Wait TRUE, ceka se na stisknuti klavesy a potom ho
  take smaze }
var aux: integer;
begin
   COff;
   gotoxy(6, 38);  textattr:= col;  write(txt);
   repeat GetChar(aux) until (aux <> kNoKey) or not Wait;
   if Wait then begin  { maze jen pokud se cekalo na klavesu }
      gotoxy(6, 38);  textattr:= 7;
      write('':length(txt));
      end;
   COn(cFull);

end;



procedure PrintHelp;
{ Vypisuje uvodni napovedu }
begin
   writeln; writeln(TITLE);
   writeln('Parametry: soubor [rozmerX] [rozmerY]');
   writeln('  soubor    - umisteni a nazev souboru s mapou');
   writeln('  [rozmerX] - sirka nove mapy (', MINMX, '-', MAXMX, ')');
   writeln('  [rozmerY] - vyska nove mapy (', MINMY, '-', MAXMY, ')');
   writeln;
end;


var EM : TEditMap;
    lm: word;
    c: integer;
    Act: byte;     { aktivni objekt }

begin
   VFInit;  { inicializace VP }
   if ParamCount = 0 then PrintHelp else begin  { osetreni parametru }
      Act:= 1;
      lm:= LastMode;  TextMode(CO80 + Font8x8); COff; { prechod na mod 80x50 }
      textattr:= 7;  clrscr;
      textattr:= 31; write('':80);  gotoxy(1, 1);
      write(TITLE, ' (', ParamStr(1), ')');
      PrintInfo(5, 40);
      PrintTable(55, 5, Act);
      if LoadMap(ParamStr(1), EM.M) = LMNoErr   { nahraje mapu }
         then begin EM.IsPlayer:= TRUE;  EM.IsExit:= TRUE; end
         else begin                             { nebo vytvori novou }
            {$R-} val(copy(ParamStr(2), 1, 2), EM.M.SizeX , c) {$R+};
            if (c <> 0) or (ParamStr(2) = '') then EM.M.SizeX:= MAXMX;
            {$R-} val(copy(ParamStr(3), 1, 2), EM.M.SizeY , c) {$R+};
            if (c <> 0) or (ParamStr(3) = '') then EM.M.SizeY:= MAXMY;
            InitMap(EM, EM.M.SizeX, EM.M.SizeY);
            end;
      PrintMap(5,5, EM);
      MovePos(EM, 0, 0);  COn(cFull);

      repeat                                   { ovladaci cyklus }
         GetChar(c);
         if c <> kNoKey then case c of
            kUp   : MovePos(EM, 0, -1);
            kDown : MovePos(EM, 0, +1);
            kLeft : MovePos(EM, -1, 0);
            kRight: MovePos(EM, +1, 0);
            kPgUp : MovePos(EM, 0, -MAXMY);
            kPgDn : MovePos(EM, 0, +MAXMY);
            kHome : MovePos(EM, -MAXMX, 0);
            kEnd  : MovePos(EM, +MAXMX, 0);
            kSpc  : PutItem(EM, Act);
            KEnt  : begin Msg(11, 'Ukladam mapu... ', FALSE);
                          if EM.IsPlayer and EM.IsExit then
                             if SaveMap(EM.M, ParamStr(1))
                                then Msg(11,'Mapa byla uspesne ulozena',TRUE)
                                else Msg(12, 'Ulozeni se nezdarilo', TRUE)
                           else Msg(12, 'Mapa neni jeste kompletni', TRUE);
                           MovePos(EM, 0, 0);
                           end;
            KBSpc : begin Msg(11, 'Nacitam mapu... ', FALSE);
                          if LoadMap(ParamStr(1), EM.M) = LMNoErr then begin
                             PrintMap(5,5, EM);
                             Msg(11, 'Mapa byla uspesne nactena', TRUE);
                             MovePos(EM, -MAXMX, -MAXMY);
                             end
                          else begin
                             Msg(12, 'Nacteni mapy se nezdarilo ', TRUE);
                             MovePos(EM, 0, 0);
                             end;
                          end;
            ord(',') : begin Act:= Act - 1;
                             if Act < 1 then Act:= LastObj;
                             PrintTable(55, 5, Act);
                             MovePos(EM, 0, 0);
                             end;
            kTab, ord('.') : begin Act:= Act + 1;
                                   if Act > LastObj then Act:= 1;
                                   PrintTable(55, 5, Act);
                                   MovePos(EM, 0, 0);
                                   end;

            end;
      until c = kEsc;

      COff;  COn(cNormal);
      TextMode(lm);
      end;
end.